#  PyGiNaC  Copyright (C) 2019 Vladimir V. Kisil
#
#  This test was adapted from check/parser_bugs.cpp as found in GiNaC
#  version 1.7.8, under the GNU General Public License.
#
#  GiNaC Copyright (C) 1999-2019 Johannes Gutenberg University Mainz, Germany
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from ginac import *
from random import randrange
import sys

def check_parser1():
    """
    >>> check_parser1()
    Basic consistency checks passed.
    """
    error = False
    # Check some string parsing
    srep="-a-b"
    reader = parser()
    e = reader(srep)
    a = reader.get_syms()["a"]
    b = reader.get_syms()["b"]
    g = - a - b
    d = (e - g).expand()
    if not d.is_zero():
        error = True
        print(srep+" erroneously parsed as %s" % str(e))
        print("Basic consistency checks failed.")
    else:
        print("Basic consistency checks passed.")

def check_parser2():
    """
    >>> check_parser2()
    Basic consistency checks passed.
    """
    error = False
    # Check some string parsing
    srep = "5-(3*x)/10"
    reader = parser()
    e = reader(srep)
    x = reader.get_syms()["x"]
    g = 5 - (3*x)/10
    d = (e - g).expand()
    if not d.is_zero():
        error = True
        print(srep+" erroneously parsed as %s" % str(e))
        print("Basic consistency checks failed.")
    else:
        print("Basic consistency checks passed.")

def check_parser3():
    """
    >>> check_parser3()
    Basic consistency checks passed.
    """
    error = False
    # Check some string parsing
    srep = "5-(2*I)/3"
    reader = parser()
    e = reader(srep)
    g = numeric(5) - (numeric(2)*I)/3
    d = (e - g).expand()
    if not d.is_zero():
        error = True
        print(srep+" erroneously parsed as %s" % str(e))
        print("Basic consistency checks failed.")
    else:
        print("Basic consistency checks passed.")

def check_parser4():
    """
    >>> check_parser4()
    Basic consistency checks passed.
    """
    error = False
    # parser happily accepted various junk like 'x^2()+1'
    junk = "x^2()+1"
    reader = parser()
    try:
        e = reader(junk)
        error = True
        print("Parser erroneously read junk '%s' as '%s'" % (junk, str(e)))
        print("Basic consistency checks failed.")
    except ValueError:
        print("Basic consistency checks passed.")
        pass

def check_parser5():
    """
    >>> check_parser4()
    Basic consistency checks passed.
    """
    error = False
    # Check methods in parser class
    srep = "x^2+1"
    reader = parser({}, True)
    try:
        e = reader(srep)
        error = True
        print("Parser erroneously read string '" + srep + "' with an unknown symbol")
        print("Basic consistency checks failed.")
    except ValueError:
        pass

    reader.set_syms({"x" : symbol("X")})
    try:
        e = reader(srep)
    except ValueError:
        error = True
        print("Parser erroneously did not read string '" + srep + "' with a known symbol")
        print("Basic consistency checks failed.")
        pass

    if not error:
        print("Basic consistency checks passed.")
    
if __name__ == "__main__":
    import doctest, sys
    doctest.testmod( sys.modules.get( __name__), verbose=True)

